﻿' Created by Ken Burkhalter (Copyright ©2013-All Rights Reserved)
' You are free to use and modify the code as you wish for personal and non-commercial uses, 
' however any distribution of this software to others shall include credit to Ken Burkhalter as the author.
'
' Release History
' [130630] v1.0.1  -released
' [130713] v1.1.0  -fixed sendReq sequence with KeepAlive=False and added proper {} syntax.
' [130714] v1.1.0.1-added "Unrecognized command User error message"
' [130717] v2.0b   -Added capture of BI Return Codes. Added STATUS cmd (reads work but writes are still being developed)
' [130731] v2.0.1  -Enhanced error msg when cmmd not recognized to help users debug.
' [130912] v2.1.0  -Added STATUS command set
' [130925] v2.2.0  -Chgd write method for rtnDataFile to try and fix file locked issue that sometimes occured.
' [130927] v2.2.1  -Added DateTime to error comments for better correlation with events.
' [131110] v2.2.2  -bug fix to correct PTZ command failure
' [131211] v2.2.3  -bug fix. Chgd TRIGGER from camconfig class to standalone TRIGGER class, as required.

' This program (jsonBI) is a bi-directional application using the JSON (JavaScript Object Notation) interface to provide methods for remote administration,
' handling both camera control and functionality of the Blue Iris (BI) Video Security and Webcam software.

' This program has no implied warranties for performance, functionality, or suitability for user's purpose.  
' It is designed as a non-showing Windows Forms Application so it runs without screen interruptions.
'
Imports System.Net
Imports System.IO
Imports System.Text
Imports System.Security.Cryptography

' To Send a json command . . . .
' Run jsonBI exe with your BI IP:port address, your user name, and password, then a semicolon ";"
' with commands and arguments following.  Multiple commands can be issued, each separated by a ";"
' The argument syntax is: command class, short camera name, action, value  (eg, camconfig DrvCam pause -1)
' Separate commands from the login with a ; (semicolon) and multiple commands from each other with the ";" also.
' 
' For example:
'   "C:/..../jsonBI.exe login http://192.168.1.90:8080/json  keb qwert1234; camconfig pause ThtrCam  -1"               [single command]
'   "C:/..../jsonBI.exe login http://192.168.1.90:8080/json  keb qwert1234; camconfig pause ThtrCam -1; ptz DrvCam 5"  [two commands]
'
' Return Data from BI is stored in C:/biRtnCodes.txt



Public Class jsonBI
    Public _response As String
    Public sessionID As String
    Public cmmd As String
    Public _IPaddr As String    'Used in all sendReq() calls



    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Dim cmdclass As String = ""
        Dim cmd1 As String = ""
        Dim cmd2 As String = ""
        Dim target As String = ""
        Dim camera As String = ""
        Dim value As String = ""
        Dim action As String = ""
        Dim user As String = ""
        Dim pw As String = ""
        Dim cmdval As String = ""
        Dim commands As String = ""
        Dim parm(5) As String
        Dim lines(25) As String
        Dim parlen As Integer
        Dim rtnData = New String(25) {}

        Try
            commands = Command$()
            'Command$ nicely Trims the input to eliminate extra chars.
            If commands = "" Then
                Exit Sub
            End If
            ' First split the Command line into individual cmmd groups between the ; chars
            lines = commands.Split(New [Char]() {";"c}, StringSplitOptions.RemoveEmptyEntries)
            ' We now have a lines() array with each array element containing a singe json command string.
            ' Now begin a loop to handle each command until done

            For Each line As String In lines
                If line <> "" Then
                    parm = line.Split(New [Char]() {" "c, " "c}, StringSplitOptions.RemoveEmptyEntries)

                    'The calling parameters, if any, are now in parm(1)... parm(4)
                    'Give them friendly names to make the camera control code easier to read.
                    'See specific Subs for calling sequences.
                    'Parm(0) is always the  command class (eg, login, camconfig, status)
                    cmdclass = parm(0)
                    parlen = parm.Length
                    If cmdclass = "login" Then
                        _IPaddr = parm(1) 'If this is a LOGIN parm(1) will hold the IP address.
                        user = parm(2)
                        pw = parm(3)
                        Dim httpOK As Boolean = _IPaddr.Contains("http://")
                        Dim jsonOK As Boolean = _IPaddr.Contains("/json")
                        Dim userOK As Boolean = Not user.Length = 0
                        Dim pwOK As Boolean = Not pw.Length = 0
                        If Not (httpOK And jsonOK And userOK And pwOK) Then
                            MsgBox("Login malformed.  Address must start with http:// abd end with /json and there must be a user ID and password.")
                        End If
                        login(user, pw)
                    ElseIf cmdclass = "logout" Then
                        logout()
                    ElseIf cmdclass = "camconfig" Then
                        If parlen = 2 Then
                            ' Simply a status request for a camera.
                            camconfig(parm(1))
                        Else
                            action = parm(1)
                            camera = parm(2) 'CamName for camera control functions
                            value = parm(3)
                            If action = "enable" Then
                                enable(camera, value)
                            ElseIf action = "motion" Then
                                motion(camera, value)
                            ElseIf action = "pause" Then
                                Pause(camera, value)
                            ElseIf action = "ptzcycle" Then
                                ptzcycle(camera, value)
                            Else : action = "ptzevents"
                                ptzevents(camera, value)
                            End If
                        End If

                    ElseIf cmdclass = "ptz" Then
                        camera = parm(1) 'CamName for camera control functions
                        value = parm(2) 'button, up/down/left/right
                        ptz(camera, value)

                    ElseIf cmdclass = "status" Then
                        If parlen = 1 Then
                            'eg: status
                            action = ""  'Null parms because this is a simple Get status
                            status(parlen, "", "", "")
                        ElseIf parlen = 3 Then  'eg, signal, profile
                            ' eg: status signal 1
                            action = parm(1)
                            cmdval = parm(2)
                            status(parlen, action, cmdval, "")
                        ElseIf parlen = 4 Then  ' dio 
                            'examples: status dio 5000 3 (Triggers Output 3 for 5000 ms) 
                            '          status dio true 2 (Forces Output 2 On)
                            action = parm(1)
                            cmd1 = parm(2)
                            target = parm(3)
                            status(parlen, action, cmd1, target)
                        Else
                            MsgBox("Status command has incorrect number of arguments.")
                        End If

                    ElseIf cmdclass = "trigger" Then
                        camera = parm(1) 'CamName for camera control functions
                        trigger(camera)
                    Else
                        MsgBox("The json command (" + cmdclass + ") is not recognized in the cmmd string . . ." + vbLf + vbLf + commands)
                    End If
                End If
            Next
            lines = _response.Split(New [Char]() {"{"c}, StringSplitOptions.RemoveEmptyEntries)
            ' Separate the Session data from the Status Data.
            Dim subLine As String = lines(lines.Length - 1)
            Dim subLineDat() As String = subLine.Split(New [Char]() {","c}, StringSplitOptions.RemoveEmptyEntries)

            Dim rtnDataLen As Integer = 0
            Dim i As Integer = 0
            Dim j As Integer = 0
            Dim k As Integer = 0
            Dim bracketDat = New String(20) {}

            For i = 0 To subLineDat.Length - 1
                If subLineDat(i) = "" Then
                    Continue For
                ElseIf Not (subLineDat(i).Contains(":[")) Then
                    ' Trim quotes(") out of strings so we have just characters
                    subLineDat(i) = subLineDat(i).Replace("""", "")
                    rtnData(j) = subLineDat(i)
                    j = j + 1
                Else
                    ' We've encountered a "[" so parse everything until the end "]"
                    ' Reset index
                    k = 0
                    ReDim bracketDat(19)
                    bracketDat(k) = subLineDat(i)
                    i = i + 1
                    k = k + 1
                    If bracketDat(0).Contains("]") Then
                        rtnData(j) = bracketDat(0)
                    Else
                        While Not subLineDat(i).Contains("]")
                            ' We've got the [ so keep incrementing until we find the ].
                            bracketDat(k) = subLineDat(i)
                            i = i + 1
                            k = k + 1
                        End While
                        bracketDat(k) = subLineDat(i)
                        rtnData(j) = String.Join(",", bracketDat)
                        ' Because bracketDat is dimensioned as 20, and the string shouldn't be that big,
                        '  trim junk off the end of the Joined string.
                    End If
                    rtnData(j) = rtnData(j).TrimEnd(",", " ")
                    ' Also trim the quotes out of the string. Replace [","] with [, ]
                    rtnData(j) = rtnData(j).Replace(""",""", ", ")
                    ' Now replace any remaining quotes with null.
                    rtnData(j) = rtnData(j).Replace("""", "")
                    j = j + 1
                End If
            Next
            ' Decrement j back to last value
            j = j - 1
            'The last entry may have trailing junk like } or spaces, so trim to just before the first "}"
            Dim ndx As Integer = rtnData(j).IndexOf("}")
            rtnData(j) = rtnData(j).Substring(0, ndx)
            rtnDataLen = j
            ReDim Preserve rtnData(rtnDataLen)

            Using writer As StreamWriter = New StreamWriter("C:/biRtnCodes.txt")
                ' Loop through return data lines
                For i = 0 To rtnDataLen
                    ' Write number with format string
                    writer.WriteLine(rtnData(i))
                Next i
            End Using

        Catch ex As Exception
            MsgBox(DateTime.Now + ex.ToString)
        End Try

        Me.Close()
    End Sub


    '
    ' Call LOGIN    /../jsonBI login, BI ip addr:port/json, user name, user pw
    '
    Public Sub login(ByVal _user As String, ByVal _pw As String)
        Try
            sendReq("""cmd""" & ":" & """login""")
            sessionID = _response
            Dim sessionIdx As Integer = sessionID.IndexOf("session")

            sessionID = sessionID.Substring(sessionIdx + 10, 32)
            Dim makeHash As String = _user & ":" & sessionID & ":" & _pw
            Dim MyMD5 As String = getMd5Hash(makeHash)

            ' sendReq will bracket the cmd resulting in a string that looks like this . . .
            ' {"cmd":"login","session":"182c8a04f7d4ab042ff8e4a2","response":"0f5dbe27396e9d6c8dc3fedf02095efd"}
            cmmd = """cmd""" & ":" & """login""" & "," & """session""" & ":" & Chr(34) & sessionID & Chr(34) & "," & """response""" & ":" & Chr(34) & MyMD5 & Chr(34)
            sendReq(cmmd)
            'MsgBox("Login Init Response: " & _response & Chr(10) & Chr(10) & "makeHash: " & makeHash & Chr(10) & Chr(10) & "Login cmd: " & cmmd & Chr(10) & Chr(10) & "Login Final Response: " & _response)
        Catch ex As Exception
            MsgBox("No Session ID was returned by the login attemp to Blue Iris")
        End Try
    End Sub

    '
    ' Call LOGOUT    /../jsonBI logout
    '

    Public Sub logout()
        sendReq("""cmd""" & ":" & """logout""" & "," & """session""" & ":" & Chr(34) & sessionID & Chr(34))
        sessionID = ""
    End Sub

    '
    ' Call CAMCONFIG    /../jsonBI  camconfig
    ' Just get status.
    Public Sub camconfig(ByVal _camera As String)
        'Just get the status of a camera
        cmmd = """cmd""" & ":" & """camconfig""" & "," & """camera""" & ":" & Chr(34) & _camera & Chr(34) & "," & """session""" & ":" & Chr(34) & sessionID & Chr(34)
        'MsgBox(cmmd)
        sendReq(cmmd)
    End Sub

    '
    ' Call ENABLE    /../jsonBI  enable, camera, true/false
    '
    Public Sub enable(ByVal _camera As String, ByVal _value As String)
        'enable:true or false enable or disable the camera
        cmmd = """cmd""" & ":" & """camconfig""" & "," & """camera""" & ":" & Chr(34) & _camera & Chr(34) & "," & """enable""" & ":" & _value & "," & """session""" & ":" & Chr(34) & sessionID & Chr(34)
        'MsgBox(cmmd)
        sendReq(cmmd)
    End Sub

    '
    ' Call TRIGGER    /../jsonBI trigger, camera
    '
    Public Sub trigger(ByVal _camera As String)
        cmmd = """cmd""" & ":" & """trigger""" & "," & """camera""" & ":" & Chr(34) & _camera & Chr(34) & "," & """session""" & ":" & Chr(34) & sessionID & Chr(34)
        sendReq(cmmd)
    End Sub

    '
    ' Call MOTION    /../jsonBI motion, camera, true/false
    '
    Public Sub motion(ByVal _camera As String, ByVal _value As String)
        'motion:true or false enable or disable the motion detector
        cmmd = """cmd""" & ":" & """camconfig""" & "," & """camera""" & ":" & Chr(34) & _camera & Chr(34) & "," & """motion""" & ":" & _value & "," & """session""" & ":" & Chr(34) & sessionID & Chr(34)
        'MsgBox(cmmd)
        sendReq(cmmd)
    End Sub

    '
    ' Call PAUSE    /../jsonBI pause, camera, value
    '
    Public Sub Pause(ByVal _camera As String, ByVal _value As String)
        '  pause:n sends a pause command, and returns a value in seconds, where n is
        '     -1: pause indefinitely
        '     0: un-pause
        '     1..3: add 30 seconds, 1 minute, 1 hour to the pause time
        cmmd = """cmd""" & ":" & """camconfig""" & "," & """camera""" & ":" & Chr(34) & _camera & Chr(34) & "," & """pause""" & ":" & _value & "," & """session""" & ":" & Chr(34) & sessionID & Chr(34)
        'MsgBox(cmmd)
        sendReq(cmmd)
    End Sub

    '
    ' Call ptzcycle    /../jsonBI ptzcycle, camera, true/false
    '
    Public Sub ptzcycle(ByVal _camera As String, ByVal _value As String)
        'ptzcycle:true or false enables or disables the preset cycle feature
        cmmd = """cmd""" & ":" & """camconfig""" & "," & """camera""" & ":" & Chr(34) & _camera & Chr(34) & "," & """ptzcycle""" & ":" & _value & "," & """session""" & ":" & Chr(34) & sessionID & Chr(34)
        'MsgBox(cmmd)
        sendReq(cmmd)
    End Sub

    '
    ' Call ptzevents    /../jsonBI ptzevents, camera, true/false
    '
    Public Sub ptzevents(ByVal _camera As String, ByVal _value As String)
        'ptzevents:true or false enables or disables the PTZ event schedule
        cmmd = """cmd""" & ":" & """camconfig""" & "," & """camera""" & ":" & Chr(34) & _camera & Chr(34) & "," & """ptzevents""" & ":" & _value & "," & """session""" & ":" & Chr(34) & sessionID & Chr(34)
        'MsgBox(cmmd)
        sendReq(cmmd)
    End Sub

    '
    ' Call ptz    /../jsonBI ptz, camera, value
    '
    Public Sub ptz(ByVal _camera As String, ByVal _value As String)
        'Operate a camera's PTZ functions.
        'camera: a camera's short name
        'button: this value determines the PTZ operation performed:
        '0:      Pan(Left)
        '1:      Pan(Right)
        '2:      Tilt(up)
        '3:      Tilt(down)
        '4: Center or home (if supported by camera)
        '5: Zoom in
        '6:      Zoom(out)
        '8..10: Power mode, 50, 60, or outdoor
        '11..26: Brightness 0-15
        '27..33: Contrast 0-6
        '34..35: IR on, off
        '101..120: Go to preset position 1..20
        'updown: Send a value of 1 to indicate that a complementary "stop" event will follow; 
        ' send 0 otherwise and the camera will be moved for a preset duration

        cmmd = """cmd""" & ":" & """ptz""" & "," & """camera""" & ":" & Chr(34) & _camera & Chr(34) & "," & """button""" & ":" & _value & "," & """session""" & ":" & Chr(34) & sessionID & Chr(34)
        'MsgBox(cmmd)
        sendReq(cmmd)
    End Sub

    '
    ' Call status    /../jsonBI status, action, cmdval, target
    ' Value can be: signal; profile; dio; play
    'Return Code Example . . .
    '  {"result":"success","session":"3b271b7325d219bd63895d621e591cb3","data":{"signal":"1","cpu":11,"mem":"215.7M","lock":"0","profile":"-1","dio":[0,0,0,0,0,0,0,0],"clips":"New: 109 [10%] +518.8G; Stored: 284 [14%] +518.8G","uptime":"0:04:11:29"}}
    '
    Private Sub status(ByVal parlen As Integer, ByVal par1 As String, ByVal par2 As String, ByVal par3 As String)
        'status: Get (and optionally set) the state of the traffic signal icon, active global profile as well as the schedule's hold/run state
        Dim timer As String = Nothing
        If parlen = 1 Then
            ' Get Status only
            cmmd = """cmd""" & ":" & """status""" & "," & """session""" & ":" & Chr(34) & sessionID & Chr(34)
        ElseIf parlen = 3 Then
            ' Signal, Profile, Play
            cmmd = """cmd""" & ":" & """status""" & "," & Chr(34) & par1 & Chr(34) & ":" & par2 & "," & """session""" & ":" & Chr(34) & sessionID & Chr(34)
        ElseIf parlen = 4 And UCase(par1) = "DIO" Then
            ' called >> status dio output 3 force true/time
            ' where: dio=par1; output=par2; 3=par3; force=par4; true/time=par5
            ' send as json string >> "cmd":"status","dio":{"output":3,"force":true},"session":"7a4d60f27c3743ea179f36db5efd2bb7"
            '          {"cmd":"status","dio":{"msec":5000,"output":0},"session":"7c8927af6d595790277e46e9609e7e38"}
            '          {"cmd":"status","dio":{"force":true,"output":0},"session":"7c8927af6d595790277e46e9609e7e38"}

            If (IsNumeric(par2)) Then
                timer = "msec"""
            Else
                timer = "force"""
            End If
            cmmd = """cmd""" & ":" & """status""" & "," & """dio"":{""" & timer & ":" & par2 & "," & """output"":" & par3 & "}," & """session""" & ":" & Chr(34) & sessionID & Chr(34)
        Else
            MsgBox(cmmd + "has incorrect parameters for status dio call")
        End If
        'MsgBox(cmmd)
        sendReq(cmmd)
    End Sub


    Public Sub sendReq(ByVal _cmd As String)

        Dim s As HttpWebRequest = WebRequest.Create(_IPaddr)
        s.Method = "POST"
        Dim postdata As String = "{" & _cmd & "}"
        Dim postdatabytes As Byte() = Encoding.UTF8.GetBytes(postdata)
        s.ContentType = "application/json"
        s.ContentLength = postdatabytes.Length
        s.KeepAlive = False

        Dim dataStream As Stream = s.GetRequestStream()
        dataStream.Write(postdatabytes, 0, postdatabytes.Length)
        dataStream.Close()

        Dim response As HttpWebResponse = s.GetResponse()
        Dim datastreamR As Stream = response.GetResponseStream()
        Dim reader As New StreamReader(datastreamR)
        _response = reader.ReadToEnd()
        reader.Close()
        datastreamR.Close()
        response.Close()

    End Sub


    Function getMd5Hash(ByVal input As String) As String

        Dim md5Hasher As New MD5CryptoServiceProvider()
        Dim data As Byte() = md5Hasher.ComputeHash(Encoding.Default.GetBytes(input))

        Dim sBuilder As New StringBuilder()

        Dim i As Integer
        For i = 0 To data.Length - 1
            sBuilder.Append(data(i).ToString("x2"))
        Next i

        Return sBuilder.ToString()

    End Function

End Class
